from datetime import datetime

import requests
from requests import ConnectTimeout, ReadTimeout

from BaseResponse import BaseResponse
from EncryptionUtil import EncryptionConfig
from EnvironmentConfig import EnvironmentConfig
from SdkResponseCode import SdkResponseCode
from Timestamp import Timestamp


class ValidateAccountOTP(object):

    def validate_account_otp(self, validate_account_otp_payload, credentials):

        try:
            get_response = EnvironmentConfig()
            if not get_response.credential_available(credentials):
                return get_response.throw_exception(status=get_response.empty_credential_code,
                                                    data=get_response.empty_credential_msg)
            else:
                rpg_environment = EnvironmentConfig.set_rpg_environment(credentials)
                headers = self.set_header(validate_account_otp_payload, credentials)
                url = rpg_environment['VALIDATE_ACC_OTP__URL']
                if not credentials.connection_timeout:
                    credentials.connection_timeout = 30000
                config = EncryptionConfig()
                validate_account_otp_payload_list = validate_account_otp_payload.auth_params
                validate_account_otp_payload_otp_value = config.AES128(credentials.enc_key, validate_account_otp_payload_list["OTP"], credentials.enc_vector)
                validate_account_otp_payload_card_value = config.AES128(credentials.enc_key, validate_account_otp_payload_list["Card"], credentials.enc_vector)

                get_otp = {
                    "param1":"OTP",
                    "value": validate_account_otp_payload_otp_value
                }

                get_card = {
                    "param2":"CARD",
                    "value": validate_account_otp_payload_card_value
                }

                get_authParams = [get_otp, get_card]

                payload = {
                    'remitaTransRef': config.AES128(credentials.enc_key, validate_account_otp_payload.remita_trans_ref,credentials.enc_vector),
                    'authParams': get_authParams
                }
                try:
                    response = requests.post(url, headers=headers, json=payload)
                    validate_account_otp_response = BaseResponse(response.content)

                except ConnectTimeout:
                    return get_response.throw_exception(status=SdkResponseCode.CONNECTION_TIMEOUT_CODE,
                                                        data=SdkResponseCode.CONNECTION_TIMEOUT)
                except ValueError:
                    return get_response.throw_exception(status=SdkResponseCode.ERROR_IN_VALUE_CODE,
                                                        data=SdkResponseCode.ERROR_IN_VALUE)
                except ReadTimeout:
                    return get_response.throw_exception(status=SdkResponseCode.CONNECTION_TIMEOUT_CODE,
                                                        data=SdkResponseCode.CONNECTION_TIMEOUT)
                except ConnectionError as e:  # This is the correct syntax
                    return get_response.throw_exception(status=SdkResponseCode.ERROR_WHILE_CONNECTING_CODE,
                                                        data=SdkResponseCode.ERROR_WHILE_CONNECTING)

            return validate_account_otp_response

        except Exception:
            return get_response.throw_exception(status=SdkResponseCode.ERROR_PROCESSING_REQUEST_CODE,
                                            data=SdkResponseCode.ERROR_PROCESSING_REQUEST)



    def set_header(self, validate_account_otp_payload, credentials):

        hash_string = credentials.api_key + validate_account_otp_payload.request_id + credentials.api_token

        txn_hash = EncryptionConfig.sha512(hash_string)
        time_stamp = Timestamp()

        headers = {'Content-Type': 'application/json', 'MERCHANT_ID':credentials.merchant_id, 'API_KEY':credentials.api_key,
                   'REQUEST_ID':validate_account_otp_payload.request_id, 'REQUEST_TS':time_stamp.dateTimeObj(dateTimeObj=datetime.now()),
                   'API_DETAILS_HASH': txn_hash}

        return headers






class SinglePaymentPayload:


    @property
    def to_bank(self):
        return self.__to_bank


    @to_bank.setter
    def to_bank(self, value):
        self.__to_bank = value

    @property
    def credit_account(self):
        return self.__credit_account


    @credit_account.setter
    def credit_account(self, value):
        self.__credit_account = value

    @property
    def narration(self):
        return self.__narration


    @narration.setter
    def narration(self, value):
        self.__narration = value

    @property
    def amount(self):
        return self.__amount


    @amount.setter
    def amount(self, value):
        self.__amount = value

    @property
    def trans_ref(self):
        return self.__trans_ref


    @trans_ref.setter
    def trans_ref(self, value):
        self.__trans_ref = value

    @property
    def from_bank(self):
        return self.__from_bank


    @from_bank.setter
    def from_bank(self, value):
        self.__from_bank = value

    @property
    def debit_account(self):
        return self.__debit_account


    @debit_account.setter
    def debit_account(self, value):
        self.__debit_account = value

    @property
    def beneficiary_email(self):
        return self.__beneficiary_email


    @beneficiary_email.setter
    def beneficiary_email(self, value):
        self.__beneficiary_email = value

    @property
    def request_id(self):
        return self.__request_id

    @request_id.setter
    def request_id(self, value):
        self.__request_id = value



from datetime import datetime

import requests
from requests import ConnectTimeout, ReadTimeout
from BaseResponse import BaseResponse
from EncryptionUtil import EncryptionConfig
from EnvironmentConfig import EnvironmentConfig
from SdkResponseCode import SdkResponseCode
from Timestamp import Timestamp


class AccountEnquiry(object):

    def account_enquiry(self, account_enquiry_payload, credentials):
        try:
            get_response = EnvironmentConfig()
            if not get_response.credential_available(credentials):
                return get_response.throw_exception(status=get_response.empty_credential_code,
                                                    data=get_response.empty_credential_msg)

            else:
                rpg_environment = EnvironmentConfig.set_rpg_environment(credentials)
                headers = self.set_header(account_enquiry_payload, credentials)
                url = rpg_environment['ACCOUNT_ENQUIRY_URL']
                if not credentials.connection_timeout:
                    credentials.connection_timeout = 30000
                config = EncryptionConfig()
                payload = {
                    'accountNo': config.AES128(credentials.enc_key, account_enquiry_payload.account_number,credentials.enc_vector),
                    'bankCode': config.AES128(credentials.enc_key, account_enquiry_payload.bank_code, credentials.enc_vector)
                }

                try:
                    response = requests.post(url, headers=headers, json=payload, timeout=credentials.connection_timeout)

                    account_enquiry_response = BaseResponse(response.content)


                except ConnectTimeout:
                    return get_response.throw_exception(status=SdkResponseCode.CONNECTION_TIMEOUT_CODE,
                                                        data=SdkResponseCode.CONNECTION_TIMEOUT)
                except ValueError:
                    return get_response.throw_exception(status=SdkResponseCode.ERROR_IN_VALUE_CODE,
                                                        data=SdkResponseCode.ERROR_IN_VALUE)
                except ReadTimeout:
                    return get_response.throw_exception(status=SdkResponseCode.CONNECTION_TIMEOUT_CODE,
                                                        data=SdkResponseCode.CONNECTION_TIMEOUT)
                except ConnectionError as e:  # This is the correct syntax
                    return get_response.throw_exception(status=SdkResponseCode.ERROR_WHILE_CONNECTING_CODE,
                                                        data=SdkResponseCode.ERROR_WHILE_CONNECTING)

            return  account_enquiry_response

        except Exception:
            return get_response.throw_exception(status=SdkResponseCode.ERROR_PROCESSING_REQUEST_CODE,
                                                data=SdkResponseCode.ERROR_PROCESSING_REQUEST)

    def set_header(self, account_enquiry_payload, credentials):

        hash_string = credentials.api_key + account_enquiry_payload.request_id + credentials.api_token

        txn_hash = EncryptionConfig.sha512(hash_string)
        time_stamp = Timestamp()

        headers = {'Content-Type': 'application/json', 'MERCHANT_ID':credentials.merchant_id, 'API_KEY':credentials.api_key,
                   'REQUEST_ID':account_enquiry_payload.request_id, 'REQUEST_TS':time_stamp.dateTimeObj(dateTimeObj=datetime.now()),
                   'API_DETAILS_HASH': txn_hash}
        return headers






from typing import List


class BulkPaymentInfo:
    total_amount: float
    batch_ref: str
    debit_account: str
    narration: str
    bank_code: str
    request_id: str


    def __init__(self, total_amount: str, batch_ref: str, debit_account: str, narration: str, bank_code: str, request_id: str) -> None:
        self.total_amount = total_amount
        self.batch_ref = batch_ref
        self.debit_account = debit_account
        self.narration = narration
        self.bank_code = bank_code
        self.request_id = request_id


class PaymentDetail:
    trans_ref: str
    narration: str
    benficiary_email: str
    benficiary_bank_code: str
    benficiary_account_number: str
    amount: float

    def __init__(self, trans_ref: str, narration: str, benficiary_email: str, benficiary_bank_code: str, benficiary_account_number: str, amount: str) -> None:
        self.trans_ref = trans_ref
        self.narration = narration
        self.benficiary_email = benficiary_email
        self.benficiary_bank_code = benficiary_bank_code
        self.benficiary_account_number = benficiary_account_number
        self.amount = amount


class BulkPaymentPayload:
    bulk_payment_info: BulkPaymentInfo
    payment_details: List[PaymentDetail]

    def __init__(self, bulk_payment_info: BulkPaymentInfo, payment_details: List[PaymentDetail]) -> None:
        self.bulk_payment_info = bulk_payment_info
        self.payment_details = payment_details



from datetime import datetime

import requests
from requests import ReadTimeout, ConnectTimeout

from BaseResponse import BaseResponse
from EncryptionUtil import EncryptionConfig
from EnvironmentConfig import EnvironmentConfig
from SdkResponseCode import SdkResponseCode
from Timestamp import Timestamp


class BulkPayment(object):

    def bulk_payment(self, bulk_payment_payload, credentials):



        try:
            get_response = EnvironmentConfig()
            if not get_response.credential_available(credentials):
                return get_response.throw_exception(status=get_response.empty_credential_code,
                                                    data=get_response.empty_credential_msg)
            else:

                rpg_environment = EnvironmentConfig.set_rpg_environment(credentials)
                headers = self.set_header(bulk_payment_payload, credentials)
                url = rpg_environment['BULK_PAYMENT_URL']
                config = EncryptionConfig()
                encrypted_payment_detail_list = []
                encrypted_bulk_payment_info = {
                    'narration': config.AES128(credentials.enc_key, bulk_payment_payload.bulk_payment_info.narration, credentials.enc_vector),
                    'totalAmount': config.AES128(credentials.enc_key, bulk_payment_payload.bulk_payment_info.total_amount, credentials.enc_vector),
                    'batchRef': config.AES128(credentials.enc_key, bulk_payment_payload.bulk_payment_info.batch_ref, credentials.enc_vector),
                    'debitAccount': config.AES128(credentials.enc_key, bulk_payment_payload.bulk_payment_info.debit_account, credentials.enc_vector),
                    'bankCode': config.AES128(credentials.enc_key, bulk_payment_payload.bulk_payment_info.bank_code, credentials.enc_vector)}

                for payment_detail_values in bulk_payment_payload.payment_details:
                    encrypted_payment_details = {}
                    encrypted_payment_details.update({
                        'transRef': config.AES128(credentials.enc_key, payment_detail_values.trans_ref, credentials.enc_vector),
                        'narration': config.AES128(credentials.enc_key, payment_detail_values.narration, credentials.enc_vector),
                        'benficiaryEmail': config.AES128(credentials.enc_key, payment_detail_values.benficiary_email, credentials.enc_vector),
                        'benficiaryBankCode': config.AES128(credentials.enc_key, payment_detail_values.benficiary_bank_code, credentials.enc_vector),
                        'benficiaryAccountNumber': config.AES128(credentials.enc_key, payment_detail_values.benficiary_account_number,
                                                      credentials.enc_vector),
                        'amount': config.AES128(credentials.enc_key, str(payment_detail_values.amount), credentials.enc_vector)})
                    encrypted_payment_detail_list.append(encrypted_payment_details)

                bulk_payment_payload.bulk_payment_info = encrypted_bulk_payment_info
                bulk_payment_payload.payment_details = encrypted_payment_detail_list


                payload = {'bulkPaymentInfo':encrypted_bulk_payment_info, 'paymentDetails':encrypted_payment_detail_list}

                try:

                    response = requests.post(url, headers=headers, json=payload, timeout=credentials.connection_timeout)
                    bulk_payment_response = BaseResponse(response.content)

                except ConnectTimeout:
                    return get_response.throw_exception(status=SdkResponseCode.CONNECTION_TIMEOUT_CODE,
                                                        data=SdkResponseCode.CONNECTION_TIMEOUT)
                except ValueError:
                    return get_response.throw_exception(status=SdkResponseCode.ERROR_IN_VALUE_CODE,
                                                        data=SdkResponseCode.ERROR_IN_VALUE)
                except ReadTimeout:
                    return get_response.throw_exception(status=SdkResponseCode.CONNECTION_TIMEOUT_CODE,
                                                        data=SdkResponseCode.CONNECTION_TIMEOUT)
                except ConnectionError as e:  # This is the correct syntax
                    return get_response.throw_exception(status=SdkResponseCode.ERROR_WHILE_CONNECTING_CODE,
                                                        data=SdkResponseCode.ERROR_WHILE_CONNECTING)

            return bulk_payment_response

        except Exception:
            return get_response.throw_exception(status=SdkResponseCode.ERROR_PROCESSING_REQUEST_CODE,
                                                data=SdkResponseCode.ERROR_PROCESSING_REQUEST)




    def set_header(self, bulk_payment_payload, credentials):

        hash_string = credentials.api_key + bulk_payment_payload.bulk_payment_info.request_id + credentials.api_token

        txn_hash = EncryptionConfig.sha512(hash_string)

        time_stamp = Timestamp()

        headers = {'Content-Type': 'application/json', 'MERCHANT_ID':credentials.merchant_id, 'API_KEY':credentials.api_key,
                   'REQUEST_ID':bulk_payment_payload.bulk_payment_info.request_id, 'REQUEST_TS': time_stamp.dateTimeObj(dateTimeObj=datetime.now()),
                   'API_DETAILS_HASH': txn_hash}
        return headers






class AccountEnquiryPayload:


    @property
    def account_number(self):
        return self.__account_number


    @account_number.setter
    def account_number(self, value):
        self.__account_number = value

    @property
    def bank_code(self):
        return self.__bank_code

    @bank_code.setter
    def bank_code(self, value):
        self.__bank_code = value

    @property
    def request_id(self):
        return self.__request_id

    @request_id.setter
    def request_id(self, value):
        self.__request_id = value



from typing import Optional, List


class AuthParam:
    param1: Optional[str]
    value: str
    param2: Optional[str]

    def __init__(self, param1: Optional[str], value: str, param2: Optional[str]) -> None:
        self.param1 = param1
        self.value = value
        self.param2 = param2


class ValidateAccountOTPPayload:
    remita_trans_ref: str
    auth_params: List[AuthParam]
    request_id: str

    def __init__(self, remita_trans_ref: str, auth_params: List[AuthParam], request_id) -> None:
        self.request_id = request_id
        self.remita_trans_ref = remita_trans_ref
        self.auth_params = auth_params



class PaymentStatusPayload:

    @property
    def request_id(self):
        return self.__request_id

    @request_id.setter
    def request_id(self, value):
        self.__request_id = value

    @property
    def trans_ref(self):
        return self.__trans_ref

    @trans_ref.setter
    def trans_ref(self, value):
        self.__trans_ref = value



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="remita-rits",
    version="1.0.3",
    author="SystemSpecs Limited",
    author_email="ipgtechnologyteam@gmail.com",
    description="Python SDK for Remita Interbank Transfer Service simple APIs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/RemitaNet/rits-sdk-python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.4'
)